#include <iostream>
#include <cstdio>
#include <vector>
#include <algorithm>

using namespace std;

const long long BASE = 1LL << 60;

struct rational {
    long long n, d;
    rational(long long n = 0, long long d = 1) : n(n), d(d) {}

    static rational norm(const rational & r) {
        long long g = gcd(r.n, r.d);
        return rational(r.n / g, r.d / g);
    }

    static long long gcd(long long a, long long b) { return b ? gcd(b, a % b) : a; }

    rational operator + (const rational& other) const {
	long long g = gcd(other.d, this->d);
        return norm(rational(this->n * (other.d / g) + other.n * (this->d / g), this->d / g * other.d));
    }

    rational operator / (const rational& other) const {
        return norm(rational(this->n * other.d, this->d * other.n));
    }
};

typedef pair<long long, long long> LongInt;

vector<int> find(vector<int> a, int M, int K, rational X) {
    const int N = a.size();

    vector<long long> B(N);
    for (int i = 0; i < N; ++i)
        B[i] = a[i] * X.d - X.n;

    vector<LongInt> sum(N + 1);
    for (int i = 0; i < N; ++i) {
        sum[i + 1].first = sum[i].first;
        sum[i + 1].second = sum[i].second + B[i];
        while (sum[i + 1].second >= BASE) {
            ++sum[i + 1].first;
            sum[i + 1].second -= BASE;
        }
        while (sum[i + 1].second < 0) {
            --sum[i + 1].first;
            sum[i + 1].second += BASE;
        }
    }

    typedef vector<int> row;
    vector<row> prev;
    prev.assign(K + 1, row(N + 1, -1));
    prev[0][0] = 0;
    for (int r = 1; r <= K; ++r) {
        LongInt best(+BASE, 0);
        int position = -1;
        for (int p = r * M; p <= N; ++p) {
            if (prev[r - 1][p - M] >= 0 && best > sum[p - M]) {
                best = sum[p - M];
                position = p - M;
            }
            if (best <= sum[p]) {
                prev[r][p] = p - position;
            }
        }
    }
/*
    cerr << X.n << '/' << X.d << endl;
    for (int i = 0; i < N; ++i)
        cerr << B[i] << ' ';
    cerr << endl;
    for (int i = 0; i <= N; ++i) {
        cerr << "(" << sum[i].first << ", " << sum[i].second << ") ";
    }
    cerr << endl;
    cerr << "* " << prev[K][N] << endl;
*/

    vector<int> answer;
    int r = K;
    int p = N;
    while (prev[r][p] > 0) {
        answer.push_back(prev[r][p]);
        p -= prev[r][p];
        --r;
    }
    reverse(answer.begin(), answer.end());
    return answer;
}

void solve() {
    int N, M, K;
    cin >> N >> M >> K;

    vector<int> a(N);
    for (int i = 0; i < N; ++i)
        cin >> a[i];

    rational L(0);
    rational R(1 << 30);

    vector<int> ans;

    while (max(L.d, R.d) <= N * (N - 1)) {
        rational middle = (L + R) / 2;

        vector<int> c = find(a, M, K, middle);
        if (c.size() == K) {
            ans = c;
            L = middle;
        } else {
            R = middle;
        }

        // cerr << middle.n << '/' << middle.d << ' ' << c.size() << endl;
    }

    for (int i = 0; i < ans.size(); ++i) {
        if (i > 0) cout << ' ';
        cout << ans[i];
    }
    cout << endl;
}

int main() {
    freopen("vuttsis.txt", "r", stdin);
    freopen("vuttval.txt", "w", stdout);

    solve();

    return 0;
}
